/* --------------------------------------------------------
 *  IR Receiver module
 *	(rx.v)			designed by Shinya KIMURA
 * -------------------------------------------------------- */

`include "my_const.v"

`define IDLE     3'b000
`define START    3'b001
`define ID       3'b010
`define DATA     3'b011
`define PARITY   3'b100
`define STOP     3'b101
`define RECOVER  3'b110

`define BIT_CLK  8'd14
`define HBIT_CLK 8'd6

module IR_receiver(clock, reset_N, s_in, myid, rd_stb, rd_full, data, err);
    input        clock;		// 31.25[KHz] == bit-rate * 16
    input        reset_N;	// active low reset
    input        s_in;		// serial input data
    input  [3:0] myid;		// assigned id code for this module
    input        rd_stb;        // data read strobe from external
    output       rd_full;       // receive data full flag
    output [7:0] data;		// received data
    output [3:0] err;		// {IDerr, PARITYerr, FRAMEerr, OVERRUNerr}

    wire       timeout;

    reg  [2:0] state;
    reg  [3:0] id;		// ID signal
    reg  [7:0] data;    	// DATA signal
    reg  [3:0] bcount;		// bit counter
    reg        rd_full;
    reg        rd_full_d;
    reg        start;
    reg  [3:0] err;		// error flags
    reg        parity;
    reg        IDerr, PARITYerr, FRAMEerr;
    reg        rx_end;
    reg  [7:0] intval;

    intvl_counter intvlc(clock, reset_N, start, intval, timeout);

  // receive data full flag
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
	    rd_full <= `OFF;
	end else if(rx_end) begin
	    rd_full <= `ON;
	end else if(rd_stb) begin
	    rd_full <= `OFF;
	end
    end

  // error flags
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
	    err <= {`OFF, `OFF, `OFF, `OFF};
	end else if(rx_end) begin
	    err <= {IDerr, PARITYerr, FRAMEerr, rd_full_d};
	end else if(rd_stb) begin
	    err <= {`OFF, `OFF, `OFF, `OFF};
	end
    end

    always @(posedge clock) begin
        rx_end    <= (state==`STOP) && timeout;
	rd_full_d <= rd_full;
    end

  // sequence control
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
	    state <= `IDLE;
            {IDerr, PARITYerr, FRAMEerr} <= {`OFF, `OFF, `OFF};
        end else if(state==`IDLE) begin
	    if(s_in) begin
	        start  <= `ON;
		intval <= `HBIT_CLK;
		state  <= `START;
	    end
	end else if(state==`START) begin        // start check
	    if(timeout) begin
	        if(s_in) begin
	            start  <= `ON;
		    intval <= `BIT_CLK;
		    bcount <= 4'd3;
		    state  <= `ID;
	        end else begin			 // it is noise
		    start  <= `OFF;
	            state  <= `IDLE;
                end
	    end else begin
	        start <= `OFF;
	    end
	end else if(state==`ID) begin           // ID stream input
	    if(timeout) begin
	        id[bcount] <= s_in;
		start  <= `ON;
		intval <= `BIT_CLK;
	        if(bcount) begin
	            bcount <= bcount - 1;
		end else begin
		    bcount <= 4'd7;
		    state <= `DATA;
		end
	    end else begin
	        start <= `OFF;
	    end
	end else if(state==`DATA) begin         // data stream input
	    if(timeout) begin
                data[bcount] <= s_in;
		start  <= `ON;
		intval <= `BIT_CLK;
	        if(bcount) begin
	            bcount <= bcount - 1;
		end else begin
		    state <= `PARITY;
		end
	    end else begin
	        start <= `OFF;
	    end
	end else if(state==`PARITY) begin
	    if(timeout) begin
	        parity <= s_in;
		start  <= `ON;
		intval <= `BIT_CLK;
		state  <= `STOP;
	    end else begin
	        start  <=`OFF;
	    end
	end else if(state==`STOP) begin
	    if(timeout) begin
	        if(s_in) begin  // stop bit error
		    FRAMEerr <= `ON;
		    state <= `RECOVER;
		end else begin
		    PARITYerr <= ~^{parity, id, data};
		    IDerr     <= (id==myid) ? `OFF : `ON;
		    state <= `IDLE;
                end
	    end else begin
	        start <= `OFF;
	    end
	end else if(state==`RECOVER) begin
	    if(!s_in) begin
	        state <= `IDLE;
	    end
	end
    end

endmodule
