/* --------------------------------------------------------
 *  self squash game module
 *	(squash.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

//  base clock = 10[MHz]

`include "my_const.v"

`define RIGHT 1'b1
`define LEFT  1'b0
`define UP    1'b1
`define DOWN  1'b0

`define RACKET_Y    18
`define RKT_WIDTH    8
`define RKT_P_MDFY  32

`define BALL_X_INIT  7
`define BALL_Y_INIT 11

`define WALL_LEFT     6'b000000
`define WALL_RIGHT    6'b111000
`define WALL_TOP      5'b00000
`define WALL_BOTTOM   5'b11000

`define AREA_L_END    6'b000001
`define AREA_R_END    6'b110111
`define AREA_T_END    5'b00001
`define AREA_B_END    5'b10111

`define	AREA_L_END_P1 6'b000010
`define	AREA_R_END_M1 6'b110110
`define	AREA_T_END_P1 5'b00010
`define	AREA_B_END_M1 5'b10110

module squash(xtal, reset_N, V_sync, x_count, y_count,
                       x_disp, y_disp, ball_speed, video_data,
                       rkt_trg, rkt_pulse);
    input        xtal; 		// 10[MHz] clock
    input        reset_N;	// active low reset
    input        V_sync;	// vertical   sync signal
    input  [8:0] x_count;	// horizontal position (0..511)
    input  [7:0] y_count;	// virtical   position (0..191)
    input        x_disp;	// horizontal display area
    input        y_disp;	// virtical   display area
    input        ball_speed;	// ball movemnt speed (TGLSW[0])
    output       video_data;	// video data
    output       rkt_trg;	// racket position one-shot trigger
    input        rkt_pulse;	// racket position pulse

    reg          v_clock, vsyncd;
    reg          v_count;
    reg          object, disp_area;
    reg    [5:0] ball_x;
    reg    [4:0] ball_y;
    reg          dir_x, dir_y;
    reg    [5:0] rkt_lx;
    wire   [5:0] rkt_rx;
    reg          rp_d, rp_dd;	// delay of racket position pulse

    wire         wall, ball, racket;
    wire         video_data;

  // game speed control (tglsw==1 -> low  speed, 
  //                          ==0 -> high speed)
    always @(posedge xtal or negedge reset_N) begin
	if(!reset_N) begin
	    v_count <= 0;
	end else if(V_sync & ~vsyncd) begin
            v_count <= v_count ^ ball_speed; 
        end
    end

  // game basic clock generator
    always @(posedge xtal) begin
        vsyncd  <= V_sync;
	v_clock <= V_sync & ~vsyncd & ~v_count;
    end

  // racket control
    // racket pulse trigger and pluse width detection
    assign rkt_trg = V_sync;
    
    always @(posedge xtal) begin
        rp_d  <= rkt_pulse;
        rp_dd <= rp_d;
    end
    
    assign rkt_pls_end = ~rp_d & rp_dd;

    // racket left side position    
    always @(posedge xtal or negedge reset_N) begin
        if(!reset_N) begin
            rkt_lx <= 6'b000000;
        end else if(rkt_pls_end) begin
            rkt_lx <= y_count[5:0] + `RKT_P_MDFY;
        end
    end
    
    // racket right side position
    assign rkt_rx = rkt_lx + `RKT_WIDTH;

  // ball movement control
    always @(posedge v_clock or negedge reset_N) begin
        if(!reset_N) begin
	    ball_x <= `BALL_X_INIT;
	    ball_y <= `BALL_Y_INIT;
	    dir_x  <= `RIGHT;
	    dir_y  <= `DOWN;
	end else begin
          // racket and ball relation check
            if((rkt_lx<=ball_x) && (ball_x<=rkt_rx)) begin
	        if(ball_y==`RACKET_Y-1 && dir_y==`DOWN) begin
	            dir_y <= `UP;    
		end else if(ball_y==`RACKET_Y+1 && dir_y==`UP) begin
	            dir_y <= `DOWN;    
		end
	    end	
          // left and right side wall relation check
	    if(ball_x==`AREA_L_END && dir_x==`LEFT) begin
	        ball_x <= `AREA_L_END_P1;
		dir_x  <=`RIGHT;
	    end else if(ball_x==`AREA_R_END && dir_x==`RIGHT) begin
		ball_x <= `AREA_R_END_M1;
		dir_x  <= `LEFT; 
	    end	else begin
	        ball_x <= ball_x + ((dir_x==`RIGHT) ? 1: -1);
	    end
          // top and bottom side wall relation check
	    if(ball_y==`AREA_T_END && dir_y==`UP) begin
	        ball_y <= `AREA_T_END_P1;
		dir_y  <=`DOWN;
	    end else if(ball_y==`AREA_B_END && dir_y==`DOWN) begin
		ball_y <= `AREA_B_END_M1;
		dir_y  <= `UP; 
	    end	else begin
	        ball_y <= ball_y + ((dir_y==`DOWN) ? 1: -1);
	    end
       	end
    end

  // video signal
    // video signal of ball
    assign ball = (x_count[8:3]==ball_x) && (y_count[7:3]==ball_y) ;

    // video signal of racket
    assign racket = ((rkt_lx<=x_count[8:3]) && (x_count[8:3]<=rkt_rx) &&
                     (y_count[7:3]==`RACKET_Y)) ;

    // video signal of wall
    assign wall = (x_count[8:3]==`WALL_LEFT) || (x_count[8:3]==`WALL_RIGHT) ||
                  (y_count[7:3]==`WALL_TOP ) || (y_count[7:3]==`WALL_BOTTOM   );

    // video signal mixer
    always @(posedge xtal) begin
        object    = wall | ball | racket;
        disp_area = x_disp & y_disp;
    end

    assign video_data = object & disp_area;

endmodule
