/* --------------------------------------------------------
 *  various type registers
 *	(register.v)		designed by Shinya KIMURA
 *  
 *     positive edge trigger
 *     positive edge trigger with synchronous reset
 *     positive edge trigger with asynchronous reset
 *     positive edge trigger with write enable and asynchronous reset
 *     high level trigger (level sensitive (or tarnsparent) latch)
 * -------------------------------------------------------- */

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//  positive edge trigger
//
module reg4(wr_stb, in_data, out_data);
    input        wr_stb;	// write strobe signal
    input  [3:0] in_data;
    output [3:0] out_data;

    reg    [3:0] out_data;

    always @(posedge wr_stb) begin
        out_data <= in_data;
    end
endmodule


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//  positive edge trigger with synchronous reset
//
module reg4srst(wr_stb, reset_N, in_data, out_data);
    input        wr_stb;	// write strobe signal
    input        reset_N;	// active low reset
    input  [3:0] in_data;
    output [3:0] out_data;

    reg    [3:0] out_data;

    always @(posedge wr_stb) begin
        if(!reset_N) begin
            out_data <= 4'b0000;
        end else begin
            out_data <= in_data;
        end
    end
endmodule


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//  positive edge trigger with asynchronous reset
//
module reg4arst(wr_stb, reset_N, in_data, out_data);
    input        wr_stb;	// write strobe signal
    input        reset_N;	// active low reset
    input  [3:0] in_data;
    output [3:0] out_data;

    reg    [3:0] out_data;

    always @(posedge wr_stb or negedge reset_N) begin
        if(!reset_N) begin
            out_data <= 4'b0000;
        end else begin
            out_data <= in_data;
        end
    end
endmodule


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//  positive edge trigger with write enable and asynchronous reset
//
module reg4enarst(wr_stb, reset_N, enable, in_data, out_data);
    input        wr_stb;	// write strobe signal
    input        reset_N;	// active low reset
    input        enable;	// active high registration enable
    input  [3:0] in_data;
    output [3:0] out_data;

    reg    [3:0] out_data;

    always @(posedge wr_stb or negedge reset_N) begin
        if(!reset_N) begin
            out_data <= 4'b0000;
        end else if(enable) begin
            out_data <= in_data;
        end /* else begin
            out_data <= out_data;	// = data hold
        end */
    end
endmodule


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//  high level trigger (level sensitive (or tarnsparent) latch)
//
module reg4lvl(wr_stb, in_data, out_data);
    input        wr_stb;	// write strobe signal
    input  [3:0] in_data;
    output [3:0] out_data;

    reg    [3:0] out_data;

    always @(wr_stb or in_data) begin
        if(wr_stb) begin
            out_data <= in_data;	// through pass
        end
    end
endmodule
